package apirtdatabase

import (
	"errors"
	"fmt"

	"git.sr.ht/~mil/mobroute/api/apirtinit"
	"git.sr.ht/~mil/mobroute/util/utillog"
	"git.sr.ht/~mil/mobsql"
	"github.com/go-playground/validator/v10"
)

// RTDatabase executes database RW manipulation allowing for: loading GTFS
// archives (via [Mobility Database] or custom URIs), creating computed tables,
// removing GTFS archives from DB, and more. Takes [DatabaseParams] with
// given database manipulation request and returns [DatabaseResponse] which
// indicates success or failure. Under-the-hood, this is a thin passthrough
// to the major RW functionality in [Mobsql], allowing the user to manipulate
// the underlying database.
//
// Note, the related [RTDatabaseq] allows for querying (RO) functionality on
// the underlying database.
//
// [Mobsql]: https://mr.lrdu.org/mobsql
// [Mobility Database]: https://mobilitydatabase.org/
func RTDatabase(runtime *apirtinit.MobrouteRuntime, params *DatabaseParams) (*DatabaseResponse, error) {
	utillog.Infoj("Running RTDatabase against input", params)

	if err := validator.New(validator.WithRequiredStructEnabled()).Struct(params); err != nil {
		return nil, fmt.Errorf("Validation failed request: %v", err)
	} else if response, err := runMobsqlFn(
		runtime.MobsqlRuntime,
		params.Op,
		params.FeedIDs,
		params.LoadCustomGTFSURI,
		params.LoadMDBGTFSDirectURLs,
		params.LoadMDBGTFSUpdate,
	); err != nil {
		return nil, err
	} else {
		return response, nil
	}
}

func runMobsqlFn(mobsqlRuntime *mobsql.MobsqlRuntime, op string, feedIDs []int, loadCustomGTFSURI string, loadMDBGTFSDirectURLs, loadMDBGTFSUpdate bool) (*mobsql.FeedOpResult, error) {
	if op == "loadmdbgtfs" {
		return mobsql.FeedLoadMDBGTFS(mobsqlRuntime, feedIDs, loadMDBGTFSDirectURLs, loadMDBGTFSUpdate)
	} else if op == "loadcustomgtfs" {
		if loadCustomGTFSURI == "" {
			return nil, errors.New("Must specify loadcustomgtfs_uri when using loadcustomgtfs op")
		}
		return mobsql.FeedLoadCustomGTFS(mobsqlRuntime, feedIDs[0], loadCustomGTFSURI)
	} else if op == "compute" {
		return mobsql.FeedCompute(mobsqlRuntime, feedIDs)
	} else if op == "purge_gtfs" {
		return mobsql.FeedPurge(mobsqlRuntime, feedIDs, mobsql.PurgeTablesOptionGTFS)
	} else if op == "purge_computed" {
		return mobsql.FeedPurge(mobsqlRuntime, feedIDs, mobsql.PurgeTablesOptionComputed)
	} else if op == "purge_all" {
		return mobsql.FeedPurge(mobsqlRuntime, feedIDs, mobsql.PurgeTablesOptionAll)
	} else {
		return nil, errors.New("Invalid op")
	}
}
