package apirtdatabase

import "git.sr.ht/~mil/mobsql"

// DatabaseParams holds parameters for the given [RTDatabase] request. The
// [RTDatabase] function and the options in these params are a thin
// passthrough to [Mobsql] (which should be referenced for in-depth doc
// on functionality).
//
// FeedIDs and Op are always required.
//
// Op should be one of
//   - loadmdbgtfs: Load the GTFS for the feed IDs from the Mobility Database
//     to the SQLite DB.
//   - loadcustomgtfs: Load the GTFS for the provided URI to the local SQLite
//     DB.
//   - compute: Create computed tables entries for the given feed IDs.
//   - purge_gtfs: Remove origin GTFS tables for the given feed IDs.
//   - purge_computed: Remove computed tables for the given feed IDs.
//   - purge_all: Remove both GTFS & computed tables entries for the
//     given feed IDs.
//
// FeedIDs are the [Mobility Database] feed ids (or custom feed ids). In the case
// of using a custom feed ID (for loadcustomgtfs), the feed ID must be
// user-specified *negative* integer. For more information see the [Feed IDs doc].
//
// LoadCustomGTFSURI is only used to define the URI when using loadcustomgtfs op
// and may refer to either an HTTP(s) (http://) URL for a remote GTFS archive
// or a File URI (file://) for a local GTFS archive to load.
//
// LoadMDBGTFSUpdate is only used when using the loadmdbgtfs op and determines
// whether the feed should be force 'full updated'. This involves
// re-downloading the Mobility DB prior to GTFS load & ignoring all caching
// systems. This is helpful in the case of feed updates.
//
// LoadMDBGTFSDirectURLs is only used when using the loadmdbgtfs op and
// determines whether the 'direct' agency URL should be used when downloading
// GTFS archives. By default / if false, a cloud-mirrored GTFS URL (provided
// by [Mobility Database]'s CI is used). The default cloud mirrored URL is
// more 'reliable' but can go out-of-date.
//
// Important Note: After running a `loadmdbgtfs` or `loadcustomgtfs` op, in
// most cases you will want to run a `compute` op. Computed data is based
// on origin GTFS data and each time updating GTFS data you should re-compute
// derived tables with `compute` op subsequently.
//
// [Mobility Database]: https://mobilitydatabase.org/
// [Mobsql]: https://mr.lrdu.org/mobsql
// [Feed IDs doc]: https://mr.lrdu.org/mobroute/doc/doc_gtfsfeeds/
type DatabaseParams struct {
	FeedIDs               []int  `json:"feed_ids" validate:"required,gt=0"`
	Op                    string `json:"op" validate:"required,oneof=status loadmdbgtfs loadcustomgtfs compute purge_gtfs purge_computed purge_all"`
	LoadCustomGTFSURI     string `json:"loadcustomgtfs_uri,omitempty"`
	LoadMDBGTFSDirectURLs bool   `json:"loadmdbgtfs_direct_urls,omitempty"`
	LoadMDBGTFSUpdate     bool   `json:"loadmdbgtfs_update,omitempty"`
}

// Databaseq holds the response for the given [RTDatabase] function. Contains
// information about whether the given [RTDatabase] call was successful and
// contains metadata about effected feeds.
//
// DatabaseResponse is an alias of [git.sr.ht/~mil/mobsql.FeedOpResult].
// Please see underlying type for further details.
type DatabaseResponse = mobsql.FeedOpResult
