// Package apirtdatabaseq contains the underlying implementation for the
// RTDatabaseq function and its types.
package apirtdatabaseq

import (
	"fmt"

	"git.sr.ht/~mil/mobroute/api/apirtinit"
	"git.sr.ht/~mil/mobroute/util/utillog"
	"git.sr.ht/~mil/mobsql"
	"github.com/go-playground/validator/v10"
)

// RTDatabaseq queries database RO to determine GTFS feed loaded status
// (resulting from [RTDatabase]) & shows metadata for unloaded and loaded
// feeds. Takes the query params in [DatabaseqParams] and returns resulting
// set of feed data in [DatabaseqResponse]. This is an RO operation as
// opposed to [RTDatabase] which may manipulate the database as RW.
// This function is helpful for querying whether a load operation, or compute
// operation was successful and also determining the current database state
// and the state of GTFS feeds. Under-the-hood, this is a thin passthrough to
// the major RO functionality in [Mobsql], allowing the user to query the
// underlying database. See [DatabaseqParams] for details on params.
//
// Note, the related [RTDatabase] allows for manipulation (RW) functionality on
// the underlying database.
//
// [Mobsql]: https://mr.lrdu.org/mobsql
//
// [Mobility Database]: https://mobilitydatabase.org/
func RTDatabaseq(runtime *apirtinit.MobrouteRuntime, params *DatabaseqParams) (*DatabaseqResponse, error) {
	utillog.Infoj("Running RTDatabaseq against input", params)

	var (
		feedIDs = []int{}
	)

	if err := validator.New(validator.WithRequiredStructEnabled()).Struct(params); err != nil {
		return nil, fmt.Errorf("Validation failed request: %v", err)
	}

	if params.FeedSearchFilter != nil {
		if filteredFeedIDs, err := mobsql.FeedsearchFilterToFeedIDs(runtime.MobsqlRuntime, params.FeedSearchFilter); err != nil {
			return nil, err
		} else {
			feedIDs = filteredFeedIDs
		}
	}
	if len(params.FeedIDs) > 0 {
		feedIDs = params.FeedIDs
	}

	if statuses, err := mobsql.FeedStatus(runtime.MobsqlRuntime, feedIDs); err != nil {
		return nil, fmt.Errorf("FeedStatus error: %v", err)
	} else {
		return &statuses, nil
	}
}
