package apirtschedule

import (
	"fmt"
	"time"

	"git.sr.ht/~mil/mobroute/api/apirtinit"
	"git.sr.ht/~mil/mobroute/dbquery"
	"git.sr.ht/~mil/mobroute/dbt"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
	"git.sr.ht/~mil/mobroute/util/utillog"
	"github.com/go-playground/validator/v10"
)

// RTSchedule queries the database for upcoming schedule departures for a
// given set of stop UIDs. Takes the departure schedule constraints defined
// in [ScheduleParams] and returns departures within the timeframe in
// a [ScheduleResponse]. Note: the logic used in this function handles
// timezones & operates identically to routing calls in terms of schedule
// extraction. This function is a good way to determine upcoming departures
// for a given set of stops. The stop UIDs may be determined using the [RTStops]
// function.
//
// Note: provided feed IDs in [ScheduleParams] should first be loaded and computed via [RTDatabase].

func RTSchedule(runtime *apirtinit.MobrouteRuntime, params *ScheduleParams) (*ScheduleResponse, error) {
	utillog.Infoj("Running RTSchedule against input:", params)

	// Defaults if unset
	params.Time = utilfuncs.Fif(params.Time != nil, params.Time, ptr(time.Now())) // E.g. now

	var (
		timeYYYYMMDD         = utilfuncs.DayYYYYMMDD(params.Time)
		_, timeUTCOffsetSecs = params.Time.Zone()
		timeSecondsFrom      = timeUTCOffsetSecs + int(utilfuncs.DaySeconds(params.Time))
		timeSecondsTo        = timeSecondsFrom + int(params.MaxSeconds)
	)

	if err := validator.New(validator.WithRequiredStructEnabled()).Struct(params); err != nil {
		return nil, fmt.Errorf("Validation failed request: %v", err)
	} else if err := dbquery.QueryFeedIDsLoadedAndComputed(runtime.DB, params.FeedIDs); err != nil {
		return nil, err
	} else if timezoneStrs, err := dbquery.LoadAgencyTimezones(
		runtime.DB, params.FeedIDs,
	); err != nil {
		return nil, fmt.Errorf("Problem querying timezones from DB: %v", err)
	} else if tzToUTCDeltasMap, err := utilfuncs.TimezoneToUTCDeltaMap(*timezoneStrs, timeYYYYMMDD); err != nil {
		return nil, fmt.Errorf("Coudln't convert timezone strings to UTC deltas: %v", err)
	} else if memConnsVerbose, err := dbquery.LoadConnectionsVerbose(
		runtime.DB, params.FeedIDs, tzToUTCDeltasMap, timeYYYYMMDD, timeSecondsFrom, timeSecondsTo,
		nil, ptr(params.StopUIDs),
	); err != nil {
		return nil, err
	} else {
		return &ScheduleResponse{
			Schedule: mapDBTConnectionsVerboseToScheduleDepartures(memConnsVerbose, params.Time),
		}, nil
	}
}

func mapDBTConnectionsVerboseToScheduleDepartures(stoptimes *[]dbt.ConnectionVerbose, beginTime *time.Time) *[]ScheduleDeparture {
	var (
		r                    = []ScheduleDeparture{}
		formatYYYYMMDD       = utilfuncs.DayYYYYMMDD(beginTime)
		formatTZUTCDeltasecs = utilfuncs.DayUTCOffsetSecs(beginTime)
	)

	for _, s := range *stoptimes {
		r = append(r, ScheduleDeparture{
			StopUID:        s.FromStopParentUID,
			RouteShortName: s.RouteShortName,
			RouteLongName:  s.RouteLongName,
			TripHeadsign:   s.TripHeadsign,

			DepartureTime: ptr(utilfuncs.CreateTimeFromYYYYMMDDZoneSecs(formatYYYYMMDD, formatTZUTCDeltasecs, s.FromStopDRUTCTime)),
		})
	}

	return &r
}
