package apirtstops

import (
	"fmt"

	"git.sr.ht/~mil/mobroute/api/apirtinit"
	"git.sr.ht/~mil/mobroute/dbquery"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
	"git.sr.ht/~mil/mobroute/util/utillog"
	"github.com/go-playground/validator/v10"
)

// RTStops queries database for GTFS stops near a given geocoordinate or all stops.
// Takes the constraints in [StopsParams] and returns the matched
// stops as [StopsResponse]. This function can be very helpful both to
// build out a 'nearby stops' functionality in end-applications or also
// the return stop UIDs may be used by [RTSchedule] which can show
// upcoming departures for each stop.
//
// Note: provided feed IDs in [StopsParams] should first be loaded and computed via [RTDatabase].
func RTStops(runtime *apirtinit.MobrouteRuntime, params *StopsParams) (*StopsResponse, error) {
	var (
		fromCoordinateKM float64    = 300000
		fromCoordinate   [2]float64 = [2]float64{0, 0}
	)
	utillog.Infoj("Running RTStops against input:", params)

	if err := validator.New(validator.WithRequiredStructEnabled()).Struct(params); err != nil {
		return nil, fmt.Errorf("Validation failed request: %v", err)
	} else {
		if params.FromCoord != nil {
			fromCoordinate = *params.FromCoord
		}
		if params.MaxDistKM != nil {
			fromCoordinateKM = *params.MaxDistKM
		}

		if err := dbquery.QueryFeedIDsLoadedAndComputed(runtime.DB, params.FeedIDs); err != nil {
			return nil, err
		} else if stopWalks, err := dbquery.LoadStopWalks(
			runtime.DB,
			params.FeedIDs,
			fromCoordinate,
			fromCoordinateKM,
			params.Glob,
			utilfuncs.Fif(params.NLimit == 0, -1, params.NLimit),
			true,
		); err != nil {
			return nil, err
		} else {
			return &StopsResponse{
				Stops: stopWalks,
			}, nil
		}
	}
}
