package main

import (
	"encoding/json"
	"flag"
	"fmt"
	"os"

	"git.sr.ht/~mil/mobroute"
)

var (
	subcmdDatabaseExamples = map[string]string{
		"Load a Mobility Database GTFS Feed":                   `mobroute database -p '{"op": "loadmdbgtfs", "feed_ids": [767]}'`,
		"Load a Mobility Database GTFS Feed (explicit update)": `mobroute database -p '{"op": "loadmdbgtfs", "feed_ids": [767], "loadmdbgtfs_update": true}'`,
		"Load a Custom GTFS Feed":                              `mobroute database -p '{"op": "loadcustomgtfs", "feed_ids": [-33], "loadcustomgtfs_uri": "http://web.mta.info/developers/data/nyct/subway/google_transit.zip"}'`,
		"Compute Routing-Optimized GTFS Derived Tables":        `mobroute database -p '{"op": "compute", "feed_ids": [767]}'`,
		"Purge GTFS Feed Data":                                 `mobroute database -p '{"op": "purge_gtfs", "feed_ids": [767]}'`,
		"Purge Computed Tables":                                `mobroute database -p '{"op": "purge_computed", "feed_ids": [767]}'`,
		"Purge All Tables":                                     `mobroute database -p '{"op": "purge_all", "feed_ids": [767]}'`,
	}
)

const (
	subcmdDatabaseDesc = `
The database subcommand provides diagnostic functionality allowing you to
alter the local SQLite database to import GTFS data (load), prepare routing
optimizated tables (compute), and remove existing data (purge). This is
essentially a passthrough to mobroute's underlying data ETL library mobsql.

After running database subcommands it's also helpful to query the database
using the databaseq command for the effected feed IDs.

See Mobsql (http://git.sr.ht/~mil/mobsql) for in-depth documentation
on functionality (which this subcommand is a passthrough to).
`

	subcmdDatabaseDocParams = `
JSON Parameters for database subcommand.

Required database parameters:
  - feed_ids: Array of integer feed IDs to use for database requests (ex: [767])
  - op: Operation to perform - may be one of "loadmdbgtfs", "loadcustomgtfs", "compute", "purge_gtfs", "purge_computed", "purge_all" (ex: "loadmdbgtfs")

Optional database parameters:
  - loadmdbgtfs_update: Used optionally when setting op to loadmdbgtfs, this flags to ignore underlying caching systems and explicitly update the Mobility DB & GTFS feed. (ex: true)
  - loadmdbgtfs_direct_urls: Used optionally when setting op to loadmdbgtfs, flags whether to use 'direct' agency URLs, if false or unset downloading uses Mobility Database cloud bucket URLs (ex: true)
  - loadcustomgtfs_uri: Used when setting op to loadcustomgtfs, this allows you to specify the URI to load for the custom GTFS archive (http:// or file://)
`
)

func cmdDatabase(flagDatabaseSet *flag.FlagSet, cfg *mobroute.MobrouteRuntimeConfig, flagDatabaseParams *string) {
	var databaseParams mobroute.DatabaseParams
	flagDatabaseSet.Usage = usageFunc("database", flagDatabaseSet, subcmdDatabaseDesc, subcmdDatabaseExamples)
	flagDatabaseSet.Parse(os.Args[2:])
	if *flagDatabaseParams == "" {
		flagDatabaseSet.Usage()
	} else if err := json.Unmarshal([]byte(*flagDatabaseParams), &databaseParams); err != nil {
		errorDie(os.Args[1], *flagDatabaseParams, err)
	} else if mobrouteRuntime, err := mobroute.RTInitialize(cfg); err != nil {
		errorDie(os.Args[1], *flagDatabaseParams, err)
	} else if commandData, err := mobroute.RTDatabase(mobrouteRuntime, &databaseParams); err != nil {
		errorDie(os.Args[1], *flagDatabaseParams, err)
	} else if jsonBytes, err := json.MarshalIndent(commandData, "", "  "); err != nil {
		errorDie(os.Args[1], *flagDatabaseParams, err)
	} else {
		fmt.Println(string(jsonBytes))
	}
}
