/*
Package mobroute is a library that allows you to load GTFS data, query
this data for stops & schedules, and perform routing requests against a
single or multiple feeds. The library  can be used to add public routing
functionality to existing map & navigation apps or for assistance in
bootstrapping new public transit routing projects for integrating GTFS
data. GTFS ingestion functionality is built-in and the library can
automatically import feeds from the [Mobility Database] specified by
[feed ID] or a custom GTFS archive can be provided. The core routing
logic is based on the [Connection-Scan-Algorithm]. GTFS data is stored in
a local SQLite database file and the library is geared toward on-device
and offline usage.

All public functionality is exposed through the single mobroute
([git.sr.ht/~mil/mobroute]) package. Functionality outside of the
top-level mobroute package should be considered to be implementation
details, unstable, and not used by end consumers. Note that types are
aliased in this package so you should look at the underlying reference
types for field members etc. However if a type is unexported that means
it is subject to change in future updates.

The library offers the following primary functionalities:

  - Load GTFS data (either via [Mobility Database] [feed ID] or via a
    user-specified file:// or http:// URI) to a local SQLite DB.
  - Compute GTFS-derived tables optimized for schedule & routing
    calculations.
  - Lookup GTFS stops near a certain geocoordinate or query all stops
    for a given feed.
  - Lookup GTFS schedules (e.g. upcoming departure times) for a given
    set of stops for a given set of feeds.
  - Perform routing calculations via the [Connection-Scan-Algorithm]
    based on input GTFS data with various tunable properties.

# Building

Ensure sqlite-dev is installed and Go >1.21

Also note: when building code that imports this package, you must pass
`-tags=sqlite_math_functions` to the `go build` command as this enables
the underlying sqlite library to utilize math functions which Mobroute's
core logic depends on. Example for build with tags below.

Import via:

	import "git.sr.ht/~mil/mobroute/v0.9.0"

Build via:

	go build -tags=sqlite_math_functions foo.go

# Overview of Functions

There are six public functions exposed from the library as a whole;
and understanding each is equivalent to grokking the library as a
whole. All functions rely on a [MobrouteRuntime] which can be created with
[RTInitialize]. The 6 public functions are:

 1. [RTInitialize]: Initialize a [MobrouteRuntime] runtime which pools a
    common SQLite database connection and is used as a param for all other
    library functions.
 2. [RTDatabase]: Executes database RW manipulation allowing for:
    loading GTFS archives (via Mobility Database or custom URIs),
    creating computed tables, removing GTFS archives from DB,
    and more.
 3. [RTDatabaseq]: Queries database RO to determine GTFS feed loaded
    status (resulting from [RTDatabase]) & shows metadata for unloaded
    and loaded feeds.
 4. [RTStops]: Queries database for GTFS stops near a given
    geocoordinate (or all stops).
 5. [RTSchedule]: Queries database for upcoming schedule (departures) for
    a given set of stop UIDs.
 6. [RTRoute]: Performs point-to-point routing calculations via the
    [Connection-Scan-Algorithm] based on GTFS data extracted from database
    queries.

The general usage pattern for using Mobroute as an library is:

  - (a) Use [RTInitialize] to create a [MobrouteRuntime].
  - (b) Load a GTFS feed into the SQLite DB via using [RTDatabase] with the `loadmdbgtfs` or `loadcustomgtfs` op.
  - (c) Then, compute GTFS-derived tables for the feed via using [RTDatabase] with the `compute` op.
  - (d) Finally, query database state with [RTDatabaseq], query stops with [RTStops], query schedules with [RTSchedule], or perform routing calculations via CSA via [RTRoute].

Please reference the below examples for general usage in which you'll find
this same pattern.

# Standalone (Runnable) Examples

Within the `examples/` folder there are a number of standalone runnable
examples. You can run each of these examples by using go run; note to make
sure to specify the sqlite_math_functions build tag.

	go run -tags=sqlite_math_functions examples/route/main.go

Currently implemented examples are as follows:

  - [Load Mobility Database GTFS Feed Example]: Load a [Mobility Database] GTFS feed to the local SQLite DB.
  - [Update Mobility Database GTFS Feed Example]: Update a [Mobility Database] GTFS feed to the local SQLite DB.
  - [Load Custom GTFS Feed Example]: Load a custom GTFS feed (either file:// or http:// URI) to the local SQLite DB.
  - [Route Example]: Perform a routing request using the Connection-Scan-Algorithm (based on loaded GTFS data).
  - [Stops Example]: Query stops (based on loaded GTFS data).
  - [Schedule Example]: Query upcoming departures (based on a loaded GTFS data).

The recommended way to get started with library usage is to simply run & adapt
these examples to your needs.

# Additional Documentation & Resources

  - [Mobroute Documentation Website]: Central documentation website
  - [System Diagrams]: Diagrams which can be very helpful for understanding general usage pattern & system architecture
  - [Transito]: A mobile app (Android & Linux) via Gio which uses Mobroute's Go library and can serve as a reference implementation.
  - [Mobroute CLI]: Provides a barebones reference implementation for all functionality offered by the library (and can also be helpful for debugging).

[Mobroute CLI]: https://mr.lrdu.org/mobroute/doc/userguide_cli/
[Transito]: http://git.sr.ht/~mil/transito
[Connection-Scan-Algorithm]: https://i11www.iti.kit.edu/extra/publications/dpsw-isftr-13.pdf
[Load Mobility Database GTFS Feed Example]: https://git.sr.ht/~mil/mobroute/tree/master/item/examples/load_mdbgtfs/main.go
[Update Mobility Database GTFS Feed Example]: https://git.sr.ht/~mil/mobroute/tree/master/item/examples/update_mdbgtfs/main.go
[Load Custom GTFS Feed Example]: https://git.sr.ht/~mil/mobroute/tree/master/item/examples/load_customgtfs/main.go
[Route Example]: https://git.sr.ht/~mil/mobroute/tree/master/item/examples/route/main.go
[Schedule Example]: https://git.sr.ht/~mil/mobroute/tree/master/item/examples/schedule/main.go
[Stops Example]: https://git.sr.ht/~mil/mobroute/tree/master/item/examples/stops/main.go
[Mobroute Documentation Website]: http://mr.lrdu.org
[System Diagrams]: http://mr.lrdu.org/mobroute/doc/doc_diagrams/
[feed ID]: https://mr.lrdu.org/mobroute/doc/doc_gtfsfeeds/
[Mobility Database]: https://mobilitydatabase.org/

[Mobroute Website]: http://mr.lrdu.org
[documentation website]: http://mr.lrdu.org
[Mobsql]: http://mr.lrdu.org/mobsql
[Mobroute]: http://mr.lrdu.org/mobroute
*/
package mobroute
